/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: _gdsl_bstree.h
 * $Revision: 1.5 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: _gdsl_bstree.h,v 1.5 2002/02/21 09:55:51 ndarnis Exp $
 */

#ifndef __GDSL_BSTREE_H_
#define __GDSL_BSTREE_H_

#include "gdsl_types.h"
#include "_gdsl_bintree.h"
#include "gdsl_macros.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef _gdsl_bintree_t _gdsl_bstree_t;

/*
 * Map node function type
 */
typedef int (* gdsl_bstree_map_func_t) (_gdsl_bstree_t, void *);

/******************************************************************************/
/* Allocation and deallocation functions                                      */
/******************************************************************************/

/*
 * Creates a new low-level binary search tree.
 * Allocates a new low-level binary search tree data structure which root's 
 * content is sets to E.
 * In case of success, returns the newly allocated _gdsl_bstree_t structure.
 * In case of failure, returns NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: nothing
 */
extern _gdsl_bstree_t
_gdsl_bstree_create (const gdsl_element_t E               /* The tree content */
		     );

/*
 * Destroy a low-level binary search tree.
 * Flush and destroy T. If FREE_F != NULL, this function is used to deallocate
 * each T's element. Otherwise nothing is done with T's element.
 *
 * Complexity: O( |T| )
 *
 * Requires: nothing
 */
extern void 
_gdsl_bstree_destroy (_gdsl_bstree_t T,                /* The tree to destroy */
		      gdsl_free_func_t FREE_F      /* The deallocate function */
		      );

/*
 * Copy a low-level binary search tree.
 * Creates and returns a copy of T using COPY_F on each element to copy them.
 * In case of success, returns the copy of T.
 * In case of failure, returns NULL.
 *
 * Complexity: O( |T| )
 *
 * Requires: COPY_F != NULL
 */
extern _gdsl_bstree_t
_gdsl_bstree_copy (const _gdsl_bstree_t T,                /* The tree to copy */
		   const gdsl_copy_func_t COPY_F         /* The copy function */
		   );

/******************************************************************************/
/* Information functions                                                      */
/******************************************************************************/

/* 
 * Check if a low-level binary search tree is empty.
 * Returns TRUE if T is empty, returns FALSE otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: nothing
 */
extern bool
_gdsl_bstree_is_empty (const _gdsl_bstree_t T            /* The tree to check */
		       );

/*
 * Check if a low-level binary search tree is reduced to a leaf.
 * Returns TRUE if T is a leaf, returns FALSE otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bstree_t
 */
extern bool
_gdsl_bstree_is_leaf (const _gdsl_bstree_t T             /* The tree to check */
		      );

/*
 * Gets the root content of a low-level binary search tree.
 * Returns the contents of T's root.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bstree_t
 */
extern gdsl_element_t
_gdsl_bstree_get_content (const _gdsl_bstree_t T           /* The tree to use */
			  );

/*
 * Check if a low-level binary search tree is a root.
 * Returns TRUE if T is a root, FALSE otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bstree_t
 */
extern bool
_gdsl_bstree_is_root (const _gdsl_bstree_t T             /* The tree to check */
		      );

/*
 * Gets the parent tree of a low-level binary search tree.
 * Returns T's parent if T isn't a root, or NULL if T is a root.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bstree_t
 */
extern _gdsl_bstree_t
_gdsl_bstree_get_parent (const _gdsl_bstree_t T            /* The tree to use */
			 );

/*
 * Gets the left sub-tree of a low-level binary search tree.
 * Returns T's left sub-tree.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bstree_t
 */
extern _gdsl_bstree_t
_gdsl_bstree_get_left_subtree (const _gdsl_bstree_t T      /* The tree to use */
			       );

/*
 * Gets the right sub-tree of a low-level binary search tree.
 * Returns T's right sub-tree.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid _gdsl_bstree_t
 */
extern _gdsl_bstree_t
_gdsl_bstree_get_right_subtree (const _gdsl_bstree_t T     /* The tree to use */
				);

/*
 * Calculate the height of a low-level binary search tree.
 * Computes and returns the height of T.
 *
 * Complexity: O( |T| )
 *
 * Requires: nothing
 */
extern ulong
_gdsl_bstree_height (const _gdsl_bstree_t T/* The tree to compute height from */
		     );

/*
 * Calculate the size of a low-level binary search tree.
 * Computes ans returns the size of T (ie. |T|).
 *
 * Complexity: O( |T| )
 *
 * Requires: nothing
 */
extern ulong
_gdsl_bstree_size (const _gdsl_bstree_t T    /* The tree to compute size from */
		  );
  
/******************************************************************************/
/* Inserting/removing functions                                               */
/******************************************************************************/

/*
 * Inserts an element in a low-level binary search tree.
 * Inserts E in T, by using COMP_F to determinate where puting E. 
 * In case of success, the newly inserted element is returned.
 * In case of failure, returns NULL.
 *
 * Complexity: O( )
 *
 * Requires: COMP_F != NULL
 */
extern gdsl_element_t
_gdsl_bstree_insert (_gdsl_bstree_t *T,            /* The tree to insert into */
		     const gdsl_compare_func_t COMP_F,    /* Compare function */
		     const gdsl_element_t E     /* The element to insert in T */
		     );

/*
 * Removes an element from a low-level binary search tree.
 * Removes the first founded element e assumed to be 'equal' to VALUE, by using 
 * COMP_F function. If e is found, it is removed from T and e is returned, 
 * otherwise NULL is returned.
 * Note: the resulting T is modified by examinating the left sub-tree from the
 *       founded e.
 *
 * Complexity: O( )
 *
 * Requires: COMP_F != NULL
 */
extern gdsl_element_t
_gdsl_bstree_remove_by_value (_gdsl_bstree_t *T,   /* The tree to remove from */
			      const gdsl_compare_func_t COMP_F, /* Comp func. */
			      void *VALUE       /* Value used to find element */
			      );

/******************************************************************************/
/* Research functions                                                         */
/******************************************************************************/

/*
 * Search for a particular element into a low-level binary search tree.
 * Research an element E in T, by using COMP_F function to find it. 
 * Return the tree containing E if it's found, NULL otherwise.
 *
 * Complexity: O( )
 *
 * Requires: COMP_F != NULL
 */
extern _gdsl_bstree_t
_gdsl_bstree_search (const _gdsl_bstree_t T,       /* The tree to search from */
		     const gdsl_compare_func_t COMP_F,/* Comp function to use */
		     void *VALUE            /* Value used to find the element */
		     );

/******************************************************************************/
/* Parse functions                                                            */
/******************************************************************************/

/*
 * Parse a low-level binary search tree in prefixed order.
 * Parse all T's nodes in prefixed order. The MAP_F function is launched on each
 * node with the USER_DATA argument. If MAP_F returns 0, then 
 * _gdsl_bstree_prefix_parse() stops and returns its last examinated node.
 * Returns NULL when the parsing is done.
 *
 * Complexity: O( |T| )
 *
 * Requires: MAP_F != NULL
 */
extern _gdsl_bstree_t
_gdsl_bstree_prefix_parse (const _gdsl_bstree_t T,       /* The tree to parse */
			   const gdsl_bstree_map_func_t MAP_F,/* Parse funct. */
			   void *USER_DATA  /* Additionnal arguments to MAP_F */
			   );

/*
 * Parse a low-level binary search tree in infixed order.
 * Parse all T's nodes in infixed order. The MAP_F function is launched on each
 * node with the USER_DATA argument. If MAP_F returns 0, then 
 * _gdsl_bstree_infix_parse() stops and returns its last examinated node.
 * Returns NULL when the parsing is done.
 *
 * Complexity: O( |T| )
 *
 * Requires: MAP_F != NULL
 */
extern _gdsl_bstree_t
_gdsl_bstree_infix_parse (const _gdsl_bstree_t T,        /* The tree to parse */
			  const gdsl_bstree_map_func_t MAP_F, /* Parse funct. */
			  void *USER_DATA   /* Additionnal arguments to MAP_F */
			  );

/*
 * Parse a low-level binary search tree in postfixed order.
 * Parse all T's nodes in postfixed order. The MAP_F function is launched on 
 * each node with the USER_DATA argument. If MAP_F returns 0, then 
 * _gdsl_bstree_postfix_parse() stops and returns its last examinated node.
 * Returns NULL when the parsing is done.
 *
 * Complexity: O( |T| )
 *
 * Requires: MAP_F != NULL
 */
extern _gdsl_bstree_t
_gdsl_bstree_postfix_parse (const _gdsl_bstree_t T,      /* The tree to parse */
			    const gdsl_bstree_map_func_t MAP_F, /* Parse func */
			    void *USER_DATA      /* Additionnal args to MAP_F */
			    );

/******************************************************************************/
/* Input/output functions                                                     */
/******************************************************************************/

/*
 * Writes the content of all nodes of a low-level binary search tree to a file.
 * Writes T's nodes contents to OUTPUT_FILE, using WRITE_F function.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |T| )
 *
 * Requires: WRITE_F != NULL 
 *           & OUTPUT_FILE != NULL
 */
extern void
_gdsl_bstree_write (const _gdsl_bstree_t T,              /* The tree to write */
		    const gdsl_write_func_t WRITE_F,       /* Write function  */
		    FILE *OUTPUT_FILE,              /* The file to write into */
		    void *USER_DATA    /* User informations passed to WRITE_F */
		    );

/*
 * Writes the content of a low-level binary search tree to a file into XML.
 * Writes T's nodes contents to OUTPUT_FILE, into XML language.
 * If WRITE_F != NULL, then uses WRITE_F to write T's nodes content to 
 * OUTPUT_FILE.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |T| )
 *
 * Requires: OUTPUT_FILE != NULL
 *
 */
extern void
_gdsl_bstree_write_xml (const _gdsl_bstree_t T,          /* The tree to write */
			const gdsl_write_func_t WRITE_F,    /* Write function */
			FILE *OUTPUT_FILE,          /* The file to write into */
			void *USER_DATA       /* User infos passed to WRITE_F */
			);

/*
 * Dumps the internal structure of a low-level binary search tree to a file.
 * Dumps T's structure to OUTPUT_FILE. 
 * If WRITE_F != NULL, then uses WRITE_F to write T's nodes content to 
 * OUTPUT_FILE.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |T| )
 *
 * Requires: OUTPUT_FILE != NULL
 */
extern void
_gdsl_bstree_dump (const _gdsl_bstree_t T,                /* The tree to dump */
		   const gdsl_write_func_t WRITE_F,         /* Write function */
		   FILE *OUTPUT_FILE,             /* The file where to dump T */
		   void *USER_DATA     /* User informations passed to WRITE_F */
		   );

/******************************************************************************/
/* Display functions                                                          */
/******************************************************************************/

/*
 * ONLY FOR TESTING: this function will be modified in the future.
 */
extern void 
_gdsl_bstree_gfx_display (const _gdsl_bstree_t T,
			  const char * (*key_display)(gdsl_element_t)
			  );

#ifdef __cplusplus
}
#endif

#endif /* _GDSL_BSTREE_H_ */
