/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: _gdsl_list.h
 * $Revision: 1.5 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: _gdsl_list.h,v 1.5 2002/02/21 09:55:51 ndarnis Exp $
 */

#ifndef __GDSL_LIST_H_
#define __GDSL_LIST_H_

#include <stdio.h>

#include "_gdsl_list_node.h"
#include "gdsl_types.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef _gdsl_list_node_t _gdsl_list_t;

/*
 * Map node function type
 */  
typedef int (*gdsl_list_map_func_t) (_gdsl_list_node_t, void *);

/******************************************************************************/
/* Management functions                                                       */
/******************************************************************************/

/*
 * Creates a low-level list.
 * Creates and allocates a new list data structure containing one node pointing 
 * to CONTENT.
 * In case of success, returns the newly allocated data structure.
 * In case of failure, returns NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: nothing
 */
extern _gdsl_list_t
_gdsl_list_create (const gdsl_element_t CONTENT  /* Content of the first node */
		   );

/* 
 * Destroys a low-level list.
 * Removes and deallocates all chained nodes starting with NODE.
 * If FREE_F != NULL, then the elements in nodes are deallocated using FREE_F 
 * on each of them. If FREE_F == NULL, elements in nodes are not modified.
 *
 * Complexity: O( |L| ), where L is the linked list of node starting by NODE
 *
 * Requires: nothing
 */
extern void 
_gdsl_list_destroy (_gdsl_list_node_t NODE,          /* First node to destroy */
		    const gdsl_free_func_t FREE_F/* Free nodes content funct. */
		    );

/******************************************************************************/
/* Information functions                                                      */
/******************************************************************************/

/* 
 * Check if a low-level list is empty.
 * Returns TRUE if the low-level list starting with NODE is empty, FALSE 
 * otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: nothing
 */
extern bool
_gdsl_list_is_empty (const _gdsl_list_node_t NODE   /* The list-node to check */
		     );

/******************************************************************************/
/* Link/unlink functions                                                      */
/******************************************************************************/

/*
 * Links two low-level lists together.
 * Links the two low-level lists starting by (respectively) NODE1 and NODE2 
 * together. 
 * After the link, NODE1's successor is NODE2 and NODE2's predecessor is NODE1.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE1 must be a valid _gdsl_list_node_t
 *           & NODE2 must be a valid _gdsl_list_node_t
 */
extern void
_gdsl_list_link (_gdsl_list_node_t NODE1,           /* The first list to link */
		 _gdsl_list_node_t NODE2           /* The second list to link */
		 );

/*
 * Unlinks two low-level lists.
 * Unlink the two low-level lists starting by (respectively) NODE1 and NODE2. 
 * After the unlink, NODE1's successor is NULL and NODE2's predecessor is NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE1 must be a valid _gdsl_list_node_t
 *           & NODE2 must be a valid _gdsl_list_node_t
 */
extern void
_gdsl_list_unlink (_gdsl_list_node_t NODE1,        /* The firt list to unlink */
		   _gdsl_list_node_t NODE2       /* The second list to unlink */
		   );

/******************************************************************************/
/* Research functions                                                         */
/******************************************************************************/

/*
 * Search for a particular node in a low-level list.
 * Search a node by comparing all L's nodes contents with VAL, using COMP_F
 * comparison function. When the first node's content 'equal' to VAL is found,
 * this function returns it. 
 * Returns NULL if L is empty or if no contents equal to VAL is found.
 *
 * Complexity: O( |L| )
 *
 * Requires: COMP_F != NULL
 */
extern _gdsl_list_node_t
_gdsl_list_search_by_value (_gdsl_list_node_t L,     /* The list to search in */
			    const gdsl_compare_func_t COMP_F, /* Comp. funct. */
			    void *VAL            /* The value to compare with */
			    );

/******************************************************************************/
/* Insert functions                                                           */
/******************************************************************************/

/* 
 * Inserts a node after another one in a low-level list.
 * Inserts NODE after PREV.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE must be a valid _gdsl_list_node_t
 *           & PREV must be a valid _gdsl_list_node_t
 */
extern void 
_gdsl_list_insert_after (_gdsl_list_node_t NODE,        /* The node to insert */
			 _gdsl_list_node_t PREV  /* The previous node of NODE */
			 );

/* 
 * Inserts a node before another one in a low-level list.
 * Inserts NODE before SUCC.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE must be a valid _gdsl_list_node_t
 *           & SUCC must be a valid _gdsl_list_node_t
 */
extern void
_gdsl_list_insert_before (_gdsl_list_node_t NODE,       /* The node to insert */
			  _gdsl_list_node_t SUCC/* The successor node of NODE */
			  );

/******************************************************************************/
/* Remove functions                                                           */
/******************************************************************************/

/*
 * Removes a node from a low-level list.
 * Removes NODE from the low-level list starting by NODE in which it is 
 * inserted. Only removes the NODE, does nothing with NODE's content but returns
 * it.
 *
 * Complexity: O( 1 )
 *
 * Requires: NODE must be a valid _gdsl_list_node_t
 */
extern gdsl_element_t
_gdsl_list_remove (_gdsl_list_node_t NODE               /* The node to remove */
		   );

/******************************************************************************/
/* Map functions                                                              */
/******************************************************************************/

/*
 * Parse a low-level list in forward order.
 * Apply MAP_F on each node starting by HEAD to the last node. If MAP_F returns
 * 0 for a particular node N, then it stops, and N is returned.
 * Otherwise, the maping continues until the last node and returns NULL.
 * The additionnal argument USER_DATA could be used to be passed to MAP_F, it 
 * will not be modified.
 *
 * Complexity: O( |L| )
 *
 * Requires: MAP_F != NULL
 */
extern _gdsl_list_node_t
_gdsl_list_map_forward (const _gdsl_list_node_t HEAD,  /* Head of list to map */
			const gdsl_list_map_func_t MAP_F, /* Mapping function */
			void *USER_DATA                         /* User infos */ 
			);
		
/*
 * Parse a low-level list in backward order.
 * Apply MAP_F on each node starting by TAIL to the first node. If MAP_F returns
 * 0 for a particular node N, then it stops, and N is returned.
 * Otherwise, the maping continues until the last node and returns NULL.
 * The additionnal argument USER_DATA could be used to be passed to MAP_F, it 
 * will not be modified.
 *
 * Complexity: O( |L| )
 *
 * Requires: MAP_F != NULL
 */    
extern _gdsl_list_node_t
_gdsl_list_map_backward (const _gdsl_list_node_t TAIL, /* Tail of list to map */
			 const gdsl_list_map_func_t MAP_F,/* Mapping function */
			 void *USER_DATA                        /* User infos */
			 );

/******************************************************************************/
/* Input/output functions                                                     */
/******************************************************************************/

/*
 * Writes the content of all nodes of a low-level list to a file.
 * Writes L's nodes contents to OUTPUT_FILE, using WRITE_F function.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |L| )
 *
 * Requires: WRITE_F != NULL 
 *           & OUTPUT_FILE != NULL
 */
extern void
_gdsl_list_write (const _gdsl_list_node_t L,             /* The list to write */
		  const gdsl_write_func_t WRITE_F,      /* The write function */
		  FILE *OUTPUT_FILE,                /* The file to write into */
		  void *USER_DATA      /* User informations passed to WRITE_F */
		  );

/*
 * Writes the nodes of a low-level list to a file into XML.
 * Writes L's nodes to OUTPUT_FILE, into XML language.
 * If WRITE_F != NULL, then uses WRITE_F to write L's nodes content to 
 * OUTPUT_FILE.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |L| )
 *
 * Requires: OUTPUT_FILE != NULL
 */
extern void
_gdsl_list_write_xml (const _gdsl_list_node_t L,         /* The list to write */
		      const gdsl_write_func_t WRITE_F,  /* The write function */
		      FILE *OUTPUT_FILE,            /* The file to write into */
		      void *USER_DATA  /* User informations passed to WRITE_F */
		      );
		     
/*
 * Dumps the internal structure of a low-level list to a file.
 * Dumps L's structure to OUTPUT_FILE. If WRITE_F != NULL, then use WRITE_F to 
 * write elements to OUTPUT_FILE.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |L| )
 *
 * Requires: OUTPUT_FILE != NULL
 */
extern void
_gdsl_list_dump (const _gdsl_list_node_t L,               /* The list to dump */
		 const gdsl_write_func_t WRITE_F,       /* The write function */
		 FILE *OUTPUT_FILE,               /* The file where to dump T */
		 void *USER_DATA       /* User informations passed to WRITE_F */
		 );

#ifdef __cplusplus
}
#endif

#endif /* __GDSL_LIST_H_ */
