/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: gdsl_2darray.h
 * $Revision: 1.4 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: gdsl_2darray.h,v 1.4 2002/02/21 09:55:51 ndarnis Exp $
 */

#ifndef _GDSL_2DARRAY_H_
#define _GDSL_2DARRAY_H_

#include <stdio.h>

#include "gdsl_types.h"
#include "gdsl_macros.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct gdsl_2darray *gdsl_2darray_t;

/*
 * Write function type.
 * FLAG1 is equal to TRUE when E is the first row element
 * FLAG2 is equal to TRUE when E is the last row element
 */
  typedef void (* gdsl_2darray_write_func_t) (gdsl_element_t E, 
					      FILE *OUTPUT_FILE,
					      bool FLAG1,
					      bool FLAG2,
					      void *USER_DATA);

/******************************************************************************/
/* Management functions                                                       */
/******************************************************************************/

/*
 * Creates a new 2D-array. 
 * Creates and returns a new 2D-array data structure with R rows and C columns 
 * which name is set to a copy of NAME.
 * In case of success, returns the newly allocated data structure.
 * In case of failure, returns NULL.
 * The two functions pointers ALLOC_F and FREE_F could be used to respectively, 
 * alloc and free elements in the list.
 * 
 * Complexity: O( 1 )
 *
 * Requires: ALLOC_F & FREE_F must be defined as explained in gdsl_types.h
 *
 * NOTES:
 * The functions pointers could be set to NULL to use the default ones:
 * - the default ALLOC_F simply returns its argument
 * - the default FREE_F does nothing
 */
extern gdsl_2darray_t
gdsl_2darray_create (const char *NAME,               /* Name of the new array */
		     const ulong R,                         /* Number of rows */
		     const ulong C,                      /* Number of columns */
		     const gdsl_alloc_func_t ALLOC_F,       /* Alloc function */
		     const gdsl_free_func_t FREE_F           /* Free function */
		     );

/*
 * Destroys a 2D-array.
 * Destroys and deallocate A. 
 * The FREE_F function is used to free elements of A, but no check is done to
 * see if an element is sets (ie. != NULL) or not. So, if an element is NULL, 
 * and FREE_F calls free(), it will causes a segmentation fault. It's up to you
 * to check if the element to free is NULL or not.
 * 
 * Complexity: O( R x C ), where R is the rows number of A, and C its columns
 *                         number
 *
 * Requires: A must be a valid gdsl_2darray_t
 */
extern void 
gdsl_2darray_destroy (gdsl_2darray_t A                /* The array to destroy */
		      );

/*
 * Gets the name of a 2D-array.
 * Returns A's name (and not a copy of it), so the returned string MUST NOT be
 * freed. 
 * 
 * Complexity: O( 1 )
 *
 * Requires: A must be a valid gdsl_2darray_t
 */
extern const char *
gdsl_2darray_get_name (const gdsl_2darray_t A             /* The array to use */
		       );

/* 
 * Sets the name of a 2D-array.
 * Deallocate the previous A's name if necessary, and chage it to a copy of 
 * GDSL_2DARRAY_NEW_NAME.
 * In case of success, returns the modified data structure.
 * In case of failure, returns NULL.
 * 
 * Complexity: O( 1 )
 *
 * Requires: A must be a valid gdsl_2darray_t
 */
extern gdsl_2darray_t
gdsl_2darray_set_name (gdsl_2darray_t A,                  /* The array to use */
		       const char *NEW_NAME              /* The new name of A */
		       );

/*
 * Gets the number of rows of a 2D-array.
 * Returns A's rows number.
 * 
 * Complexity: O( 1 )
 *
 * Requires: A must be a valid gdsl_2darray_t
 */
extern ulong
gdsl_2darray_get_rows_count (const gdsl_2darray_t A       /* The array to use */
			     );

/*
 * Gets the number of columns of a 2D-array.
 * Returns A's columns number.
 * 
 * Complexity: O( 1 )
 *
 * Requires: A must be a valid gdsl_2darray_t
 */
extern ulong
gdsl_2darray_get_columns_count (const gdsl_2darray_t A    /* The array to use */
				);

/* 
 * Gets an element from a 2D-array.
 * Returns element of A, in row R and column C.
 * 
 * Complexity: O( 1 )
 *
 * Requires: A must be a valid gdsl_2darray_t
 *           & R <= gdsl_2darray_get_rows_count( A )
 *           & C <= gdsl_2darray_get_columns_count( A )
 */
extern gdsl_element_t
gdsl_2darray_get_content (const gdsl_2darray_t A,         /* The array to use */
			  const ulong R,         /* Row of the element to get */
			  const ulong C       /* Column of the element to get */
			  );

#define gdsl_2darray_get_element(A,R,C) gdsl_2darray_get_content(A,R,C)

/* 
 * Modify an element in a 2D-array.
 * Changes the element in row R and column C of A to E, and returns it.
 * The new element to insert is allocated using A's alloc function on KEY.
 * 
 * Complexity: O( 1 )
 *
 * Requires: A must be a valid gdsl_2darray_t
 *           R <= gdsl_2darray_get_rows_count( A )
 *           C <= gdsl_2darray_get_columns_count( A )
 */
extern gdsl_element_t
gdsl_2darray_set_content (gdsl_2darray_t A,               /* The array to use */
			  const ulong R,             /* Row where inserting E */
			  const ulong C,          /* Column where inserting E */
			  void *KEY                  /* The element to set in */
			  );

#define gdsl_2darray_set_element(A,R,C,E) gdsl_2darray_set_content (A,R,C,E)

/******************************************************************************/
/* Input/output functions                                                     */
/******************************************************************************/

/*
 * Writes the content of a 2D array to a file.
 * Writes T's nodes contents to OUTPUT_FILE, using WRITE_F function.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( R x C ), where R is the rows number of A, and C its columns
 *                         number
 *
 * Requires: A must be a valid gdsl_2darray_t
 *           & WRITE_F != NULL 
 *           & OUTPUT_FILE != NULL
 */
extern void
gdsl_2darray_write (const gdsl_2darray_t A,               /* The array to use */
		    const gdsl_2darray_write_func_t WRITE_F,  /* Write funct. */
		    FILE *OUTPUT_FILE,              /* The file to write into */
		    void *USER_DATA           /* User infos passed to WRITE_F */
		    );

/*
 * Writes the content of a 2D array to a file into XML.
 * Writes all A's elements to OUTPUT_FILE, into XML language.
 * If WRITE_F != NULL, then uses WRITE_F to write A's elements to OUTPUT_FILE.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( R x C ), where R is the rows number of A, and C its columns
 *                         number
 *
 * Requires: A must be a valid gdsl_2darray_t
 *           & OUTPUT_FILE != NULL
 *
 */
extern void
gdsl_2darray_write_xml (const gdsl_2darray_t A,           /* The array to use */
			const gdsl_write_func_t WRITE_F,    /* Write function */
			FILE *OUTPUT_FILE,          /* The file to write into */
			void *USER_DATA       /* User infos passed to WRITE_F */
			);

/*
 * Dumps the internal structure of a 2D array to a file.
 * Dumps A's structure to OUTPUT_FILE. 
 * If WRITE_F != NULL, then uses WRITE_F to write A's elements to OUTPUT_FILE.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( R x C ), where R is the rows number of A, and C its columns
 *                         number
 *
 * Requires: A must be a valid gdsl_2darray_t
 *           & OUTPUT_FILE != NULL
 */
extern void
gdsl_2darray_dump (const gdsl_2darray_t A,                /* The array to use */
		   const gdsl_write_func_t WRITE_F,         /* Write function */
		   FILE *OUTPUT_FILE,               /* The file to write into */
		   void *USER_DATA            /* User infos passed to WRITE_F */
		   );

#ifdef __cplusplus
}
#endif

#endif /* _GDSL_2DARRAY_H_ */
