/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: gdsl_bstree.h
 * $Revision: 1.4 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: gdsl_bstree.h,v 1.4 2002/02/21 09:55:51 ndarnis Exp $
 */

#ifndef _GDSL_BSTREE_H_
#define _GDSL_BSTREE_H_

#include <stdio.h>

#include "gdsl_types.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct gdsl_bstree *gdsl_bstree_t;

/******************************************************************************/
/* Allocation and deallocation functions                                      */
/******************************************************************************/

/*
 * Creates a new binary search tree.
 * Allocates a new binary search tree data structure which name is set to a copy
 * of NAME, unless NAME is NULL.
 * In case of success, returns the newly allocated data structure.
 * In case of failure, returns NULL.
 * The tree function pointers ALLOC_F, FREE_F and COMP_F could be used to
 * respectively, alloc, free and compares elements in the tree.
 *
 * Complexity: O( 1 )
 *
 * Requires: nothing
 *
 * NOTES:
 * The functions pointers could be set to NULL to use the default ones:
 * - the default ALLOC_F simply returns its argument
 * - the default FREE_F does nothing
 * - the default COMP_F always returns 0
 */
extern gdsl_bstree_t
gdsl_bstree_create (const char *NAME,                 /* Name of the new tree */
		    const gdsl_alloc_func_t ALLOC_F,        /* Alloc function */
		    const gdsl_free_func_t FREE_F,           /* Free function */
		    const gdsl_compare_func_t COMP_F   /* Comparison function */
		    );

/*
 * Destroys a binary search tree.
 * Destroys all the elements of T by calling T's FREE_F function passed to
 * gdsl_bstree_create(). Deallocate T's name and destroy T itself too.
 *
 * Complexity: O( |T| )
 *
 * Requires: T must be a valid gdsl_bstree_t
 */
extern void
gdsl_bstree_destroy (gdsl_bstree_t T                   /* The tree to destroy */
		     );

/*
 * Flush a binary search tree.
 * Destroys all the elements of T by calling T's FREE_F function passed to
 * gdsl_rbtree_create().
 * T is not deallocated itself and T's name is not modified.
 *
 * Complexity: O( |T| )
 *
 * Requires: T must be a valid gdsl_bstree_t
 * 
 */
extern void
gdsl_bstree_flush (gdsl_bstree_t T                       /* The tree to flush */
		  );

/* 
 * Sets the name of a binary search tree.
 * Deallocate the previous T's name if necessary, and change it to a copy of
 * NEW_NAME.
 * In case of success, returns the modified data structure.
 * In case of failure, returns NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid gdsl_bstree_t
 */
extern gdsl_bstree_t
gdsl_bstree_set_name (gdsl_bstree_t T,   /* The tree the name must be changed */
		      const char *NEW_NAME                   /* New name of T */
		      );

/******************************************************************************/
/* Information functions                                                      */
/******************************************************************************/

/* 
 * Gets the name of a binary search tree.
 * Returns T's name (and not a copy of it).
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid gdsl_bstree_t
 */
extern char *
gdsl_bstree_get_name (const gdsl_bstree_t T                /* The tree to use */
		      );

/*
 * Checks if a binary search tree is empty.
 * Returns TRUE if T is empty, FALSE otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid gdsl_bstree_t
 */
extern bool
gdsl_bstree_is_empty (const gdsl_bstree_t T                /* The tree to use */
		      );

/*
 * Gets the root of a binary search tree.
 * Returns root element of T.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid gdsl_bstree_t
 */
extern gdsl_element_t
gdsl_bstree_get_root (const gdsl_bstree_t T                /* The tree to use */
		      );

/*
 * Returns the size of a binary search tree.
 * Returns the size of T.
 *
 * Complexity: O( 1 )
 *
 * Requires: T must be a valid gdsl_bstree_t
 */
extern ulong
gdsl_bstree_get_size (const gdsl_bstree_t T                /* The tree to use */
		      );

/*
 * Calculate the height of a binary search tree.
 * Computes and returns the height of T.
 *
 * Complexity: O( |T| )
 *
 * Requires: T must be a valid gdsl_bstree_t
 */
extern ulong
gdsl_bstree_height (const gdsl_bstree_t T                  /* The tree to use */
		    );

/******************************************************************************/
/* Inserting/removing functions                                               */
/******************************************************************************/

/*
 * Inserts an element in a binary search tree.
 * Inserts an element into T.
 * In case of success, the inserted element is returned.
 * In case of failure, returns NULL.
 *
 * Complexity: 
 *
 * Requires: T must be a valid gdsl_bstree_t
 */
extern gdsl_element_t
gdsl_bstree_insert (gdsl_bstree_t T,                 /* The tree to insert in */
		    void *VAL   /* Value used to create new element to insert */
		    );

/*
 * Removes an element from a binary search tree.
 * Removes the first founded element E assumed to be 'equal' to VAL, by using 
 * T's COMP_F function. If E is found, it is removed from T and E is returned, 
 * otherwise NULL is returned.
 * Note: the resulting T is modified by examinating the left sub-tree from the
 *       founded E.
 *
 * Complexity: 
 *
 * Requires: T must be a valid gdsl_bstree_t
 */
extern gdsl_element_t
gdsl_bstree_remove_by_value (gdsl_bstree_t T,      /* The tree to remove from */
			     void *VAL          /* Value used to find element */ 
			     );

/*
 * Destroy an element from a binary search tree.
 * Removes the first founded element E assumed to be 'equal' to VAL, by using 
 * T's COMP_F function. If E is found, it is removed from T and E is deallocated
 * using T's FREE_F function passed to gdsl_bstree_create(), then T is returned,
 * otherwise NULL is returned.
 * Note: the resulting T is modified by examinating the left sub-tree from the
 *       founded E.
 *
 * Complexity: 
 *
 * Requires: T must be a valid gdsl_bstree_t
 */
extern gdsl_bstree_t
gdsl_bstree_destroy_by_value (gdsl_bstree_t T,    /* The tree to destroy from */
			      void *VAL         /* Value used to find element */ 
			      );

/******************************************************************************/
/* Search functions                                                           */
/******************************************************************************/

/*
 * Search for a particular element into a binary search tree.
 * Research an element E in T, using COMP_F and VAL to find it. If 
 * COMP_F == NULL, then the COMP_F function passed to gdsl_bstree_create() is 
 * used instead.
 * Return E if it's found, NULL otherwise.
 *
 * Complexity: 
 *
 * Requires: T must be a valid gdsl_bstree_t
 */
extern gdsl_element_t
gdsl_bstree_search (const gdsl_bstree_t T,           /* The tree to search in */
		    const gdsl_compare_func_t COMP_F, /* Comparison function  */
		    void *VAL               /* Value used to find the element */
		    );

/******************************************************************************/
/* Parse functions                                                            */
/******************************************************************************/

/*
 * Parse a binary search tree in prefixed order.
 * Parse T in prefixed order. The MAP_F function is launched on each element 
 * with the USER_DATA argument. If MAP_F returns 0, then 
 * gdsl_bstree_prefix_parse() stops and returns its last examinated element.
 * Returns NULL when the parsing is done.
 *
 * Complexity: O( |T| )
 *
 * Requires: T must be a valid gdsl_bstree_t
 *           & MAP_F != NULL
 */
extern gdsl_element_t
gdsl_bstree_prefix_parse (const gdsl_bstree_t T,         /* The tree to parse */
			  const gdsl_map_func_t MAP_F,  /* The parse function */
			  void *USER_DATA   /* Additionnal arguments to MAP_F */
			  );

/*
 * Parse a binary search tree in infixed order.
 * Parse T in infixed order. The MAP_F function is launched on each element 
 * with the USER_DATA argument. If MAP_F returns 0, then 
 * gdsl_bstree_infix_parse() stops and returns its last examinated element.
 * Returns NULL when the parsing is done.
 *
 * Complexity: O( |T| )
 *
 * Requires: T must be a valid gdsl_bstree_t
 *           & MAP_F != NULL
 */
extern gdsl_element_t
gdsl_bstree_infix_parse (const gdsl_bstree_t T,          /* The tree to parse */
			 const gdsl_map_func_t MAP_F,   /* The parse function */
			 void *USER_DATA    /* Additionnal arguments to MAP_F */
			 );

/*
 * Parse a binary search tree in postfixed order.
 * Parse T in postfixed order. The MAP_F function is launched on each element 
 * with the USER_DATA argument. If MAP_F returns 0, then 
 * gdsl_bstree_postfix_parse() stops and returns its last examinated element.
 * Returns NULL when the parsing is done.
 *
 * Complexity: O( |T| )
 *
 * Requires: T must be a valid gdsl_bstree_t
 *           & MAP_F != NULL
 */
extern gdsl_element_t
gdsl_bstree_postfix_parse (const gdsl_bstree_t T,        /* The tree to parse */
			   const gdsl_map_func_t MAP_F, /* The parse function */
			   void *USER_DATA  /* Additionnal arguments to MAP_F */
			   );

/******************************************************************************/
/* Input/output functions                                                     */
/******************************************************************************/

/*
 * Writes the content of a binary search tree to a file.
 * Writes T's nodes contents to OUTPUT_FILE, using WRITE_F function.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |T| )
 *
 * Requires: T must be a valid gdsl_bstree_t
 *           & WRITE_F != NULL 
 *           & OUTPUT_FILE != NULL
 */
extern void
gdsl_bstree_write (const gdsl_bstree_t T,                /* The tree to write */
		   const gdsl_write_func_t WRITE_F,         /* Write function */
		   FILE *OUTPUT_FILE,               /* The file to write into */
		   void *USER_DATA     /* User informations passed to WRITE_F */
		   );

/*
 * Writes the content of a binary search tree to a file into XML.
 * Writes T's nodes contents to OUTPUT_FILE, into XML language.
 * If WRITE_F != NULL, then uses WRITE_F to write T's nodes contents
 * to OUTPUT_FILE.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |T| )
 *
 * Requires: T must be a valid gdsl_bstree_t
 *           & OUTPUT_FILE != NULL
 *
 */
extern void
gdsl_bstree_write_xml (const gdsl_bstree_t T,            /* The tree to write */
		       const gdsl_write_func_t WRITE_F,     /* Write function */
		       FILE *OUTPUT_FILE,           /* The file to write into */
		       void *USER_DATA        /* User infos passed to WRITE_F */
		       );

/*
 * Dumps the internal structure of a binary search tree to a file.
 * Dumps T's structure to OUTPUT_FILE. If WRITE_F != NULL, then uses WRITE_F to
 * write nodes elements to OUTPUT_FILE.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |T| )
 *
 * Requires: T must be a valid gdsl_bstree_t
 *           & OUTPUT_FILE != NULL
 */
extern void
gdsl_bstree_dump (const gdsl_bstree_t T,                  /* The tree to dump */
		  const gdsl_write_func_t WRITE_F,          /* Write function */
		  FILE *OUTPUT_FILE,              /* The file where to dump T */
		  void *USER_DATA      /* User informations passed to WRITE_F */
		  );

/******************************************************************************/
/* Display functions                                                          */
/******************************************************************************/

/*
 * ONLY FOR TESTING: this function will be modified in the future.
 */
extern void 
gdsl_bstree_gfx_display (const gdsl_bstree_t T,
			 const char * (*key_display) (gdsl_element_t)
			 );
			 
#ifdef __cplusplus
}
#endif

#endif /* _GDSL_BSTREE_H_ */

