/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: gdsl_rbtree.h
 * $Revision: 1.3 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: gdsl_rbtree.h,v 1.3 2002/02/21 09:55:51 ndarnis Exp $
 */

#ifndef _GDSL_RBTREE_H_
#define _GDSL_RBTREE_H_

#include "gdsl_types.h"
#include "_gdsl_bintree.h"
#include "gdsl_macros.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct gdsl_rbtree *gdsl_rbtree_t;

/******************************************************************************/
/* Allocate and deallocate functions                                          */
/******************************************************************************/

/*
 * Creates a new red-black tree.
 * Allocates a new red-black tree data structure which name is set to a copy of
 * NAME, unless NAME is NULL.
 * In case of success, returns the newly allocated data structure.
 * In case of failure, returns NULL.
 * The tree function pointers ALLOC_F, FREE_F and COMP_F could be used to
 * respectively, alloc, free and compares elements in the tree.
 *
 * Complexity: O( 1 )
 *
 * Requires: nothing
 *
 * NOTES:
 * The functions pointers could be set to NULL to use the default ones:
 * - the default ALLOC_F simply returns its argument
 * - the default FREE_F does nothing
 * - the default COMP_F always returns 0
 */
extern gdsl_rbtree_t
gdsl_rbtree_create (const char *NAME,                 /* Name of the new tree */
		    gdsl_alloc_func_t ALLOC_F,      /* Alloc function pointer */
		    gdsl_free_func_t FREE_F,         /* Free function pointer */
		    gdsl_compare_func_t COMP_F    /* Compare function pointer */
		    );

/*
 * Destroy a red-blac tree.
 * Destroy all the elements of T by calling T's FREE_F function passed to
 * gdsl_rbtree_create(). Deallocate T's name and destroy T itself too.
 *
 * Complexity: O( |T| )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create()
 */
extern void 
gdsl_rbtree_destroy (gdsl_rbtree_t T                   /* The tree to destroy */
		     );

/*
 * Flush a red-black tree.
 * Destroy all the elements of T by calling T's FREE_F function passed to
 * gdsl_rbtree_create().
 * T is not deallocated itself and T's name is not modified.
 *
 * Complexity: O( |T| )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create()
 * 
 */
extern void 
gdsl_rbtree_flush (gdsl_rbtree_t T                       /* The tree to flush */
		   );

/* Sets the name of a red-black tree.
 * Deallocate the previous T's name if necessary, and change it to a copy of
 * NEW_NAME.
 * In case of success, returns the modified data structure.
 * In case of failure, returns NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create()

 */
extern gdsl_rbtree_t
gdsl_rbtree_set_name (gdsl_rbtree_t T,       /* The tree to get the name from */
		      const char *NEW_NAME                   /* New name of T */
		      );

/******************************************************************************/
/* Information functions                                                      */
/******************************************************************************/

/*
 * Gets the name of a red-black tree.
 * Returns T's name (and not a copy of it).
 *
 * Complexity: O( 1 )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create()
 */
extern char *
gdsl_rbtree_get_name (gdsl_rbtree_t T        /* The tree to get the name from */
		      );

/* 
 * Checks if a red-black tree is empty.
 * Returns TRUE if T is empty, FALSE otherwise.
 *
 * Complexity: O( 1 )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create()
 */
extern bool
gdsl_rbtree_is_empty (gdsl_rbtree_t T               /* The gdsl_rbtree to use */
		      );

/*
 * Gets the root of a red-black tree.
 * Returns root element of T.
 *
 * Complexity: O( 1 )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create()
 */
extern gdsl_element_t
gdsl_rbtree_get_root (gdsl_rbtree_t T        /* The tree to get the root from */
		      );

/*
 * Returns the size of a red-black tree.
 * Returns the size of T.
 *
 * Complexity: O( 1 )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create()
 */
extern ulong
gdsl_rbtree_get_size (gdsl_rbtree_t T           /* The tree to calculate size */
		      );

/*
 * Calculate the height of a red-black tree.
 * Computes and returns the height of T.
 *
 * Complexity: O( |T| )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create()
 */
extern ulong
gdsl_rbtree_height (gdsl_rbtree_t T           /* The tree to calculate height */
		    );

/******************************************************************************/
/* Inserting/removing functions                                               */
/******************************************************************************/

/*
 * Inserts an element in a red-black tree.
 * Inserts an element into T.
 * In case of success, the inserted element is returned.
 * In case of failure, returns NULL.
 *
 * Complexity: O( log( |T| ) )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create()
 */
extern gdsl_element_t
gdsl_rbtree_insert (gdsl_rbtree_t T,                 /* The tree to insert in */
		    void *VAL                   /* The element to insert in T */
		    );

/*
 * Removes a particular element from a red-black tree.
 * Removes the first founded element E assumed to be 'equal' to VAL, by using 
 * T's COMP_F function. If E is found, it is removed from T and E is returned, 
 * otherwise NULL is returned.
 *
 * Complexity: O( log( |T| ) )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create()
 */
extern gdsl_element_t
gdsl_rbtree_remove_by_value (gdsl_rbtree_t T ,     /* The tree to remove from */
			     void *VAL /* Val. used to find element to remove */
			     );

/*
 * Destroy a particular element from a red-black tree.
 * Removes the first founded element E assumed to be 'equal' to VAL, by using 
 * T's COMP_F function. If E is found, it is removed from T and deallocated 
 * using FREE_F function passed to gdsl_rbtree_create(). If E is found, T is 
 * returned, otherwise NULL is returned.
 *
 * Complexity: O( log( |T| ) )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create()
 */
extern gdsl_rbtree_t
gdsl_rbtree_destroy_by_value (gdsl_rbtree_t T ,   /* The tree to destroy from */
			      void *VAL/* Val. used to find elemnt to destroy */
			      );

/******************************************************************************/
/* Research functions                                                         */
/******************************************************************************/

/*
 * Search for a particular element into a red-black tree.
 * Research an element E in T, using COMP_F and VAL to find it. If 
 * COMP_F == NULL, then the COMP_F function passed to gdsl_rbtree_create() is
 * used instead.
 * Return E if it's found, NULL otherwise.
 *
 * Complexity: O( log( |T| ) )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create()
 */
extern gdsl_element_t
gdsl_rbtree_search (gdsl_rbtree_t T,                 /* The tree to search in */
		    gdsl_compare_func_t COMP_F, /* Comparison function to use */
		    void *VAL               /* Value used to find the element */
		    );

/******************************************************************************/
/* Parse functions                                                            */
/******************************************************************************/

/*
 * Parse a red-black tree in prefixed order.
 * Parse T in prefixed order. The MAP_F function is launched on each element 
 * with the USER_DATA argument. If MAP_F returns 0, then 
 * gdsl_rbtree_prefix_parse() stops and returns its last examinated element.
 * Returns NULL when the parsing is done.
 *
 * Complexity: O( |T| )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create() & MAP_F != NULL
 */
extern gdsl_element_t
gdsl_rbtree_prefix_parse (gdsl_rbtree_t T,               /* The tree to parse */
			  gdsl_map_func_t MAP_F,        /* The parse function */
			  void *USER_DATA   /* Additionnal arguments to MAP_F */
			  );

/*
 * Parse a red-blacktree in infixed order.
 * Parse T in infixed order. The MAP_F function is launched on each element 
 * with the USER_DATA argument. If MAP_F returns 0, then 
 * gdsl_rbtree_infix_parse() stops and returns its last examinated element.
 * Returns NULL when the parsing is done.
 *
 * Complexity: O( |T| )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create() & MAP_F != NULL
 */
extern gdsl_element_t
gdsl_rbtree_infix_parse (gdsl_rbtree_t T,                /* The tree to parse */
			 gdsl_map_func_t MAP_F,         /* The parse function */
			 void *USER_DATA    /* Additionnal arguments to MAP_F */
			 );

/*
 * Parse a red-black tree in postfixed order.
 * Parse T in postfixed order. The MAP_F function is launched on each element 
 * with the USER_DATA argument. If MAP_F returns 0, then 
 * gdsl_rbtree_postfix_parse() stops and returns its last examinated element.
 * Returns NULL when the parsing is done.
 *
 * Complexity: O( |T| )
 *
 * Requires: T successfully allocated by gdsl_rbtree_create() & MAP_F != NULL
 */
extern gdsl_element_t
gdsl_rbtree_postfix_parse (gdsl_rbtree_t T,              /* The tree to parse */
			   gdsl_map_func_t MAP_F,       /* The parse function */
			   void *USER_DATA  /* Additionnal arguments to MAP_F */
			   );

/******************************************************************************/
/* Input/output functions                                                     */
/******************************************************************************/
/*
 * Writes the content of a red-black tree to a file.
 * Writes T's nodes contents to OUTPUT_FILE, using WRITE_F function.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |T| )
 *
 * Requires: WRITE_F != NULL & OUTPUT_FILE != NULL
 */
extern void
gdsl_rbtree_write (gdsl_rbtree_t T,                      /* The tree to write */
		   gdsl_write_func_t WRITE_F,    /* Function to write content */
		   FILE *OUTPUT_FILE,               /* The file to write into */
		   void *USER_DATA     /* User informations passed to WRITE_F */
		   );

/*
 * Writes the content of a red-black tree to a file into XML.
 * Writes T's nodes contents to OUTPUT_FILE, into XML language, using WRITE_F
 * function. Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |T| )
 *
 * Requires: WRITE_F != NULL & OUTPUT_FILE != NULL
 *
 */
extern void
gdsl_rbtree_write_xml (gdsl_rbtree_t T,                  /* The tree to write */
		       gdsl_write_func_t WRITE_F,/* Function to write content */
		       FILE *OUTPUT_FILE,           /* The file to write into */
		       void *USER_DATA        /* User infos passed to WRITE_F */
		       );

/*
 * Dumps the internal structure of a red-black tree to a file.
 * Dumples T's structure to OUTPUT_FILE. If WRITE_F != NULL, then uses WRITE_F
 * to write nodes elements to OUTPUT_FILE.
 * Additionnal USER_DATA argument could be passed to WRITE_F.
 *
 * Complexity: O( |T| )
 *
 * Requires: OUTPUT_FILE != NULL
 */
extern void
gdsl_rbtree_dump (gdsl_rbtree_t T,                        /* The tree to dump */
		  gdsl_write_func_t WRITE_F,/* Function used to write content */
		  FILE *OUTPUT_FILE,              /* The file where to dump T */
		  void *USER_DATA      /* User informations passed to WRITE_F */
		  );

/******************************************************************************/
/* Display functions                                                          */
/******************************************************************************/

/*
 * ONLY FOR TESTING: this function will be modified in the future.
 */
extern void 
gdsl_rbtree_gfx_display (gdsl_rbtree_t T, 
			 char * (*key_display) (gdsl_element_t)
			 );

#ifdef __cplusplus
}
#endif

#endif /* _GDSL_RBTREE_H_ */
